/*-----------------------------------------------*/
/* queue service module                          */
/*  provides common service functions based on   */
/*  queue input                                  */
/*-----------------------------------------------*/
/* This program:                                 */
/*  (1) creates a global input queue             */
/*  (2) waits for elements to be entered to      */
/*      the queue                                */
/*  (3) processes each element as a transaction  */
/*      of the form:                             */
/*       tod outputqueue trantype trandata       */
/*  (4) repeat (2)-(3) until QUIT received       */
/*-----------------------------------------------*/
/* Output from the transaction is pushed into    */
/* the queue of name outputqueue                 */
/*                                               */
/* The first response line contains:             */
/*   status: message                             */
/* status = 0 if successful transaction          */
/*          nn if error                          */
/*-----------------------------------------------*/
/* Valid transactions are:                       */
/*   QUIT ---> stop processing                   */
/*   GETALL ---> return all members              */
/*   GETREGISTERED [n] --> get registered        */
/*                         starting with n       */
/*   REGISTER name ------> register name         */
/*   ADDMEMBER name -----> add member to list    */
/*   FLUSH --------------> flush registered      */
/*                         names to output file  */
/*   SETDRAW ----> initialize for drawing        */
/*   PICKWINNER -> pick random winner            */
/*   GETWINNERS -> get all winners               */
/*-----------------------------------------------*/
/* Revisions:                                    */
/*-----------------------------------------------*/
/* 06-15-1997 subroutine for member list check   */
/*            add choose winner functions        */
/*-----------------------------------------------*/

waitIntv = 1                            /* test every few seconds */
maxCt    = 25000                        /* repeat count (max)     */
rptCt    = 0                            /* repeat count (current) */
bkupIntv = 60                           /* backup interval        */
bkupNext = 0                            /* next backup            */
bkupIndex = 0                           /* backup start index     */
rFileMem = "members.txt"                /* initial input file     */
rFileReg = "reg.txt"                    /* registration file      */
mainq    = "rafqm"                      /* name of service queue  */
rcOK     = "0:"                         /* successful status      */

call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'     /* we need this eventually */
call SysLoadFuncs

Call RXQUEUE "Create", mainq            /* create the main queue */


/* Perform Other initialization functions before entering queue loop */

fp = STREAM(rFileReg, "c", "query exists")
IF fp <> "" THEN
    "del " rFileReg                     /* Delete file if exists */
bkupNext = bkupIntv                     /*  first backup         */

regMem.0 = 0
winPool.0 = 0
winMem.0 = 0
bkupIndex = 0
Call LoadMembers

/* Now perform queue loop and process transactions     */
/*  until told to quit                                 */

oldq  = RXQUEUE('Set', mainq)           /*  use the new queue    */
bContinue = 1
rptCt = 0

DO WHILE bContinue > 0

   DO WHILE QUEUED() > 0                /* if any queue elements */
      PARSE PULL qTime qName transdata  /*  get next element     */
      SAY qTime qName transdata         /*  echo to window       */
      Call RXQUEUE "Set", qName         /*  set output queue     */
      Call processTrans                 /*  process the trans    */
      Call RXQUEUE "Set", mainq         /*  reset to input queue */
      IF (rcq > 0) THEN
        SAY time() qName transdata "==> not successful, rc="rcq
   END

   IF (bContinue > 0) THEN
     Call SysSleep waitIntv             /* if none, delay        */

   rptCt = rptCt + 1                    /* safety valve          */
   IF (rptCt >= maxCt) THEN
     bContinue = 0

   IF rptCt == bkupNext THEN            /* if backup needed      */
     DO
        bkupNext = bkupNext + bkupIntv  /*  reschedule next      */
        Call bkupReg                    /*  and do it            */
     END

END

call RXQUEUE "Delete", mainq            /* termination logic     */
call RXQUEUE "Set", oldq

SAY "program terminating"

Return

/*-----------------------------------------------*/
/* LoadMembers procedure                         */
/*  loads members file into stem variable        */
/*-----------------------------------------------*/

loadMembers:

  rc = STREAM(rFileMem, "c", "open read")
  iLineCt = 0
  allMem.0  = 0
  DO WHILE LINES(rFileMem) > 0
     iLine = LINEIN(rFileMem)
     iLineCt = iLineCt + 1
     allMem.iLineCt = iLine
     SAY "adding member: " iLine
     allMem.0 = iLineCt
  END

  rc = STREAM(rFileMem, "c", "close")
  SAY allMem.0 " members loaded"

  Return

/*-----------------------------------------------*/
/* Procedure to process the requested transaction*/
/* ARG is the input transaction                  */
/*-----------------------------------------------*/

processTrans:

  PARSE VAR transdata reqtype reqdata
  reqtype = TRANSLATE(reqtype)
  rcq     = "0"

  IF (reqtype == "QUIT") THEN               /* QUIT request         */
    DO
       bContinue = 0                        /* set end of input     */
       Call bkupReg                         /* flush last members   */
       PUSH rcq":QUIT Accepted"             /* return message       */
       RETURN
    END

  IF (reqtype == "GETALL") THEN             /* GETALL request       */
    DO
       DO ix = allMem.0 TO 1 BY -1          /* return all members   */
          PUSH allMem.ix
       END
       PUSH rcq":" || allMem.0              /* return count         */
    RETURN
    END

  IF (reqtype == "GETREGISTERED") THEN      /* GETREGISTERED [n] req */
    DO
       IF (regMem.0 == 0) THEN              /* If none, return none  */
         DO
           rcq = "10"
           PUSH rcq":no registered members"
           RETURN
         END

       IF reqdata == "" THEN                /* If no start index,    */
          reqdata = 1                       /*  use 1                */

       IF reqdata <= regMem.0 THEN          /* If valid start index, */
          iy = reqdata                      /*  use it               */
       ELSE
          iy = 1                            /* Else use 1            */

       DO ix = regMem.0 TO iy BY -1         /* Return range of       */
          PUSH regMem.ix                    /*  registered names     */
       END
       iz = regMem.0 - iy + 1
       PUSH rcq":" || iz
    RETURN
    END

  IF (reqtype == "REGISTER") THEN           /* REGISTER name         */
    DO
       regname = reqdata

       nFound = findMember(regname)         /* For security, verify  */
       IF (nFound == 0) THEN                /*  valid member         */
         DO
            rcq = "11"
            PUSH rcq":registration rejected: " regname " not a member"
            RETURN
         END

       DO ix = 1 TO regMem.0                /*  If already registered*/
          IF (regMem.ix == regname) THEN    /*   return error msg    */
            DO
               rcq = "12"
               PUSH rcq":registration rejected: " regname " already registered"
               RETURN
            END
       END
       ix = regMem.0 + 1                    /* If not, register now  */
       regMem.0 = ix
       regMem.ix = regname
       PUSH rcq":"ix " " regname " registered "   /* Return index also     */
       RETURN
    END

  IF (reqtype == "FLUSH") THEN              /* FLUSH                 */
    DO
       IF (regMem.0 == 0) THEN              /*  If none, return      */
          DO
            rcq = "13"
            PUSH rcq":No registered members to flush"
            RETURN
          END

       fp = STREAM(rFileReg, "c", "query exists")
       IF fp <> "" THEN
         "del " rFileReg                    /* Delete file if exists */

       rc = STREAM(rFileReg, "c", "open write")
       DO ix = 1 TO regMem.0                /* Write names to file   */
          rc = LINEOUT(rFileReg, regMem.ix)
          PUSH regMem.ix " added to registered file"
       END
       PUSH rcq":"regMem.0
       rc = STREAM(rFileReg, "c", "close")
       regMem.0 = 0
       bkupIndex = regMem.0
       RETURN
    END

  IF (reqtype == "ADDMEMBER") THEN          /* ADDMEMBER             */
    DO
       ix = findMember(reqdata)
       IF (ix > 0) THEN
          DO
             rcq = 14
             PUSH rcq": "reqdata "not added, already in member list"
             RETURN
          END

       ix = allMem.0 + 1
       allMem.0 = ix
       allMem.ix = reqdata
       PUSH rcq": "reqdata "added as new member"
       RETURN
    END

  IF (reqtype == "SETDRAW") THEN
    DO
       winPool.0 = 0
       winMem.0 = 0
       DO ix = 1 TO regMem.0
          winPool.0 = ix
          winPool.ix = regMem.ix
       END
       PUSH rcq": "winPool.0 " members entered in drawing"
       RETURN
  END

  IF (reqtype == "PICKWINNER") THEN
    DO
       ix = RANDOM(1, winPool.0)
       nWinner = winPool.ix
       iz = winMem.0 + 1
       winMem.0 = iz
       winMem.iz = nWinner
       DO iz = ix+1 TO winPool.0
          iz2 = iz-1
          winPool.iz2 = winPool.iz
       END
       winPool.0 = winPool.0 - 1
       PUSH rcq":"nWinner
       RETURN
    END

  IF (reqtype == "GETWINNERS") THEN         /* GETREGISTERED [n] req */
    DO
       IF (winMem.0 == 0) THEN              /* If none, return none  */
         DO
           rcq = "15"
           PUSH rcq":no selected winners"
           RETURN
         END

       DO ix = winMem.0 TO 1 BY -1          /* Return range of       */
          PUSH winMem.ix                    /*  winning names        */
       END
       PUSH rcq":" || winMem.0
    RETURN
    END

  rcq = "99"
  PUSH rcq":Invalid transaction " reqtype

  Return

/*-----------------------------------------------*/
/* Procedure to backup the registered members    */
/*   bkupIndex = last index backed up            */
/*-----------------------------------------------*/

bkupReg:

  IF bkupIndex >= regMem.0 THEN              /* if no additions since last backup, */
    RETURN                                   /*   then return                      */

  rc = STREAM(rFileReg, "c", "open write")
  DO ix = bkupIndex+1 TO regMem.0              /* Write names to file   */
     rc = LINEOUT(rFileReg, regMem.ix)
     SAY regMem.ix " written to file "
  END
  rc = STREAM(rFileReg, "c", "close")

  bkupIndex = regMem.0

  RETURN

/*-----------------------------------------------*/
/* findMember procedure                          */
/*-----------------------------------------------*/

findMember:

  PARSE ARG memName

  DO it1 = 1 TO allMem.0
     IF (allMem.it1 == memName) THEN
        return it1
  END

  return 0

